package gstests.tutorials;

/*
Using AssertJ Swing to do Java GUI testing (of swing classes)
- Obtain from: https://search.maven.org/search?q=assertj-swing-junit
- Documentation: https://joel-costigliola.github.io/assertj/assertj-swing.html

Alternatives to AssertJSwing (googled: automate java interface testing) suggested at
https://sqa.stackexchange.com/questions/18554/open-source-tools-for-automation-of-java-gui-application-testing

Event Dispatch Thread (EDT) pages:
- https://joel-costigliola.github.io/assertj/assertj-swing-edt.html
- https://web.archive.org/web/20120526191520/http://alexruiz.developerblogs.com/?p=160
- https://web.archive.org/web/20130218063544/http://weblogs.java.net/blog/alexfromsun/archive/2006/02/debugging_swing.html

Got AssertJ Swing from Maven Central Repository:
https://search.maven.org/search?q=assertj-swing-junit
-> http://central.maven.org/maven2/org/assertj/assertj-core/3.8.0/
More jar files: http://repo1.maven.org/maven2/org/assertj/
- http://repo1.maven.org/maven2/org/assertj/assertj-swing/3.8.0/
- http://central.maven.org/maven2/org/assertj/assertj-core/3.8.0/

API:
https://joel-costigliola.github.io/assertj/swing/api/index.html

JUNIT:
- https://junit.org/junit4/faq.html#atests_2
   How do I use a test fixture?
- https://junit.org/junit4/faq.html#organize_3
   How can I run setUp() and tearDown() code once for all of my tests?

*/

// Junit imports
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

// GLI imports
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.GathererProg; // main GLI class we'll be testing
import org.greenstone.gatherer.Dictionary; // access to display strings

// Java GUI testing with AssertJ Swing
import org.assertj.swing.junit.testcase.AssertJSwingJUnitTestCase;
import org.assertj.swing.fixture.*;
import org.assertj.swing.edt.GuiActionRunner;
import org.assertj.swing.core.*;
import org.assertj.swing.data.Index;

// Selenium
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.firefox.FirefoxDriver;

// Helper classes for selenium and AssertJ Swing tests
import org.greenstone.gsdl3.testing.GSTestingUtil;
import org.greenstone.gsdl3.testing.GSGUITestingUtil;

// Java imports
import javax.swing.*;


     
// static imports
import static org.assertj.swing.launcher.ApplicationLauncher.*;
import static org.assertj.swing.finder.WindowFinder.findFrame;

import static org.greenstone.gsdl3.testing.GSGUITestingUtil.*;

public class RunGLITest extends AssertJSwingJUnitTestCase {

    private static WebDriver _driver = new FirefoxDriver(); // selenium
    
    private FrameFixture window;

    // Selenium
    @Before
    public void init()
    {
	//https://stackoverflow.com/questions/38676719/selenium-using-java-the-path-to-the-driver-executable-must-be-set-by-the-webdr
	// GS3's build.xml would have set the webdriver.gecko.driver path System.Property to
	// the location of Firefox' geckodriver executable when launching this test class already.
	// So now we can continue to just do:
	_driver.get(System.getProperty("SERVERURL"));
    }
    
    @Override
    protected void onSetUp() {
	GathererProg frame = GuiActionRunner.execute(() -> new GathererProg());           
	// IMPORTANT: note the call to 'robot()'
	// we must use the Robot from AssertJSwingJUnitTestCase
	
	//window = new FrameFixture(robot(), frame);
	//window.show(); // shows the frame to test
	
	// Launch GathererProg.java's main() method
	// See https://joel-costigliola.github.io/assertj/assertj-swing-launch.html
	
	String GSDLOS = System.getenv("GSDLOS");
	String GSDLHOME = System.getenv("GSDLHOME");
	String GSDL3HOME = System.getenv("GSDL3HOME");
	String GSDL3SRCHOME = System.getenv("GSDL3SRCHOME");
	application("org.greenstone.gatherer.GathererProg").withArgs(
								 "-gsdl", GSDLHOME,
								 "-gsdlos", GSDLOS,
								 "-gsdl3", GSDL3HOME,
								 "-gsdl3src", GSDL3SRCHOME,
								 "-testing_mode").start();
    }
    
    
  @Test
  public void testGLIRunning() {

      
      // waiting 2 seconds for window, so we can see it
      try{
	Thread.sleep(5000);
      } catch(Exception e) {
	  e.printStackTrace();
      }
      
    
      System.err.println("@@@ First test: GLI Running");
      
      
      String expectedWindowTitle = Gatherer.PROGRAM_NAME;

      window = findFrame("GUIManager").using(robot());
      	
	String gatherPaneLabel = Dictionary.get("GUI.Gather");
	System.err.println("@@@ Expecting label: " + gatherPaneLabel);

	System.err.println("@@@ Second test: that Gather panel is selected and has right title");

	JTabbedPaneFixture tab = window.tabbedPane("GUIManager.tab_pane");

	
	tab.requireSelectedTab(Index.atIndex(1));
	tab.requireTitle(gatherPaneLabel, Index.atIndex(1));

	// attempt to switch to enrich pane, uses static methods of GSGUITestingUtil
	// through static import of that class
	switchToPane(window, DOWNLOAD_PANE);

	loadCollection(window, "lucene-jdbm-demo");

	// wait a couple of seconds again?
	 try{
	     Thread.sleep(5000);
	 } catch(Exception e) {
	     e.printStackTrace();
	 }
      
	
  }

    // Selenium
    // called once and only once: to quit the firefox driver geckodriver
    @AfterClass
    public static void destroy()
    {
	_driver.quit();
    }
}
